#include <iostream>
#include <system_error>

#include "../wavelib/include/file.h"
#include "SampleFilter.h"

//////////////////////////////////////////////////////////////////////////////
// The audio filter. Uses the SampleFilter routines generated by the web site
//////////////////////////////////////////////////////////////////////////////

void audio_filter( int* left_sample, int* right_sample) {
  static SampleFilter left_filter, right_filter ;
  SampleFilter_put(&left_filter, *left_sample) ;
  *left_sample = SampleFilter_get(&left_filter) ;
  SampleFilter_put(&right_filter,*right_sample) ;
  *right_sample = SampleFilter_get(&right_filter) ;
  }

//////////////////////////////////////////////////////////////////////////////
// The main program 
// accept to arguments : the input file name, the output file name
//////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[]) {
 
  // check arguments (test <infile> <outfile>)
  if (argc != 3) {
    std::cout << "Expected 2 arguments" << std::endl;
    return 1;
  }

  // read infile's content
  wave::File read_file;
  wave::Error err = read_file.Open(argv[1], wave::kIn);
  if (err) {
    std::cout << "Something went wrong in in open" << std::endl;
    return 1;
  }

  // The waveform is stored in a vector of floats 
  // in the range ]-1,1[
  std::vector<float> content;

  err = read_file.Read(&content);
  if (err) {
    std::cout << "Something went wrong in read" << std::endl;
    return 2;
  }

  // Check wave infos
  // number of samples
  int frame_number = read_file.frame_number() ;
  std::cout << "Frame number:" << frame_number << std::endl ;
  std::cout << "Sample rate:" << read_file.sample_rate() << std::endl ;
  std::cout << "bits_per_sample:" << read_file.bits_per_sample() << std::endl ;
  std::cout << "channel_number:" << read_file.channel_number() << std::endl ;

  // Split Left and Right channel and convert to integer values
  // As the initial samples are 16 bits samples we
  // simply multiply the floating point values by 65536
  std::vector<int> left_content, right_content ;
  bool parity=true;
  for (auto sample: content) {
      parity = !parity ;
      if(parity) {
          left_content.push_back (sample*65536);
      } else  {
          right_content.push_back (sample*65536);
      }
  }

  // Filter Left and Right channel using the audio filter
  for(std::size_t i = 0; i < frame_number; ++i) {
        audio_filter(&left_content[i], &right_content[i]) ;
  }

  // Merge Left and Right channel, and regenerate floating
  // point values for the libwave library
  std::vector<float> content_out ;
  for(std::size_t i = 0; i < left_content.size(); ++i) {
      content_out.push_back( left_content[i]/65536.0)  ;
      content_out.push_back(right_content[i]/65536.0) ;
  }


  // write that same content to another file
  wave::File write_file;
  err = write_file.Open(argv[2], wave::kOut);
  if (err) {
    std::cout << "Something went wrong in out open" << std::endl;
    return 3;
  }
  write_file.set_sample_rate(read_file.sample_rate());
  write_file.set_bits_per_sample(read_file.bits_per_sample());
  write_file.set_channel_number(read_file.channel_number());

  err = write_file.Write(content_out);
  if (err) {
    std::cout << "Something went wrong in write" << std::endl;
    return 4;
  }

  return 0;
}

